function v = pnbm(u, t)
% PNBM cdf of the supremum of the absolute value of the Brownian Motion
%
% Syntax
%
% v = pnbm(t,u)
%
% Description
% PNBM(U,T) computes the cumulative distribution function of the supremum
% of the absolute value of the Brownian motion in the interval [0,T] 
% at each of the values in U.
% The input parameter T must be a positive scalar. The values in U must
% be positive.
% When the parameter T is not present we take T = 1.
%
% Reference 
% 1- "An Introduction to probability theory and its Applications Vol II" by 
%     Feller 1970 p.342
% 2- "Empirical processes with applications to statistics"
%    by Shorack, Galen R. and Wellner, Jon A. 1986.
%    expression (7) page 34
% 3- "A toolbox note on the distribution of the maximum of Gaussian process" 
%    by Azais, J.M. & Lozada-Chang, L.
  
% Version 2012/10/01
% Li-Vang Lozada Chang

M = 8;  % to truncate the infinite sum 
if nargin < 2,
    t = 1;
end

ut = u/sqrt(t);
f1 = 4/pi*exp(-pi^2./(8*ut.^2));
v = f1;
k = 1;
while abs(f1)> 10^(-M)
    K = 2*k+1;
    f1 = 4/(pi*K)*(-1)^k * exp(-K^2*pi^2./(8*ut.^2));
    v = v + f1;
    k = k + 1;
end
  
end
