function prob = pnbp(U, T, n)
% PNBP cdf of the supremum of the absolute value of a Bessel process
%
% Syntax
%
% v = pnbp(u,t,n)
% v = pnbp(u,t)
%
% Description
% PNBP(U,T,N) computes the cumulative distribution function of the supremum
% of the absolute value of a normalized Bessel process of order N defined 
% on the interval [1,T] at each of the values in U.
% The input parameter T must be a scalar greater than 1. The values in U must
% be positive. 
% When the parameter N is not present we take n = 1.
% When the parameter T is not present we take T = 2.
%
% Reference 
% 1- "Crossing probabilities for a square root boundary by a Bessel
%     process" by DeLong, David M 
%     Commun. Stat., Theory Methods A, 1981.
% 2- "A toolbox note on the distribution of the maximum of Gaussian process" 
%    by Azais, J.M. & Lozada-Chang, L.
% 

% Version 2012/10/05 (REVISAR diferentes resultados a Delong cuando C=1,DF=1)
% Li-Vang Lozada-Chang
%

if nargin < 3,
    n = 1;     
    if nargin < 2,
        T = 2;
    end
end
if T <= 1,
    disp('Error value: T must be greater than 1')
    prob = [];
    return
end

global Q bigC
Q = n/2;

prob = zeros(size(U));
i = 1;
for c = U,
    bigC = -c^2/2;
    term = 1;
    beta = -1;
    
    while (abs(term)>1e-4)
        beta = zeroHGF(beta);
        aux1 = hypergeom(beta + Q, Q + 1, bigC);
        aux2 = derivativeDelongM(beta);
        alph = -(exp(bigC)*(-bigC)^Q *aux1 )/(gamma(Q+1)*beta*aux2);
        term = alph*T^(-beta);
        prob(i) = prob(i) + term;
    end
    i = i+1;
end

% NESTED FUNCTIONS
%--------------------------------------------------------------------------
function beta = zeroHGF(x0)
% Given the parameters of the confluent hypergeometric function
% computes the first zero bigger than x0 of the equation
% M(x+q, q, -c^2/2). See DeLong81 page 2203.
% q and c are passed as global parameters.
%
% See  Estrella(2003) "Critical values and p values of Bessel process 
% distributions: computation and application to structural break tests."
% Econometric Theory}, 19(6):1128--1143, 2003. 
%
betaPrev = x0;
beta = x0;
while (abs(beta-betaPrev)<1/2)
   x0 = x0 + 1;
   beta = fzero(@delongM, x0);
end

function y = delongM(x)
% Function on Delong 81 page 2203
global Q bigC
y = hypergeom(x+Q, Q, bigC);

function Dev = derivativeDelongM(x)
% Approximation of the derivative in Delong's expression page 2203
% 
Dev = (delongM(x+1e-8)-delongM(x-1e-8))/(2e-8);


