function V = psc(U, time, omega)
% PSC cdf of the maximum of the Sine-Cosine process on [0,T]
% 
% Syntax
%    p = psc(u,t,omega)
%    p = psc(u,t)
%    p = psc(u)
%
% Description 
% PSC(U,T,OMEGA) computes the cumulative distribution function of 
% the Sine-Cosine process with parameter OMEGA on [0,T]. T must be a 
% positive scaalar 
% The default value of T is 1.
% The default value of OMEGA is 1.
%
% Reference:
% "Level Sets and Extrema of Random Processes and Fields" by
% Jean-Marc Azais, Mario Wschebor (2009). Exercise 4.1 p.130
% "A toolbox note on the distribution of the maximum of Gaussian process" 
% by Azais, J.M. & Lozada-Chang, L.  

% Version 2012/10/05 
%
%            
if nargin < 3,
    omega = 1;
    if nargin < 2,
        time = 1;
    end
end
cond = sum(omega/pi*time>[1 2]);
switch cond
     case 0
         V = ncdf(U) - time*omega/(2*pi)*exp(-U.^2/2);
     case 1
         V = 0*U;
         k = 1;
         for u = U,
            V(k) = quadgk(@(t) fun(t), pi, time*omega);
            k = k + 1;
         end
        V = ncdf(U) - time*omega/(2*pi)*exp(-U.^2/2) + V/(2*pi);
         % corrige JMA 15/06/2017
     case 2
         V = 1 - exp(-U.^2/2);
end 
% -------------------------------------------------------------
% NESTED FUNCTIONS
% -------------------------------------------------------------
    function y = ncdf(x)
        % standard normal cdf
        y = erfc(-x ./ sqrt(2))/2;
    end

    function y = fun(t)
        % integrand function 
        if abs(t-pi) < 1e-10, % avoids null denominator around pi
            y = exp(-u^2/2);
        elseif abs(2*pi - t) < 1e-10, % avoids null denominator around 2*pi
            y = 0;
        else
            y = exp(-u^2*(1-cos(t))./sin(t).^2);
        end
    end
end % function psc