function q = qbp(P, T)
% QBP inverse of CDF of the supremum of the Bessel Process order 1
%
% Syntax
%   q = qbp(p,t)
%   q = qbp(p)
%
% Description
% QBP(Q,T) computes the inverse of the cumulative distribution function of
% the supremum of the absolute value of the Normalized Bessel Process in 
% the interval [0,T] at each of the values in P.
% The input parameter T must be a positive scalar. The values in P must
% probability values.
% The default value of parameter T is 2.
%
% REMARK    c < 5
%
% Reference
% 1- "Crossing probabilities for a square root boundary by a Bessel
%     process" by DeLong, David M
%     Commun. Stat., Theory Methods A, 1981.
% 2- "A toolbox note on the distribution of the maximum of Gaussian process"
%    by Azais, J.M. & Lozada-Chang, L.
%
% Version 2012/10/11
% Li-Vang Lozada-Chang

if nargin < 2,
    T = 2;
end

q = 0*P;
options = optimset('TolX', 1e-6, 'Display', 'off');
pk = 1;
for p = P
    q(pk) = fzero(@(c) pbp(c), [0.01 3.8], options);
    pk = pk + 1;
end

% NESTED FUNCTION ------------------------------------------
    function v = pbp(c)
        term = 1;
        nu = 0;
        v = 0;
        while (abs(term) > 1e-7)
            nu = zeroDw(nu,-c);
            aux1 = Dw(nu-1,-c)*npdf(c);
            aux2 = nu*derivativeDw(nu,-c);
            term = -T^(-nu/2)*aux1/aux2;
            v = v + term;
        end
        v = v - p;
        
        function nu = zeroDw(nu0,x)
            nuPrev = nu0;
            nu = nu0;
            while (abs(nu-nuPrev) < 1e-3)
                nu0 = nu0 + 0.5;
                nu = fzero(@(nu) Dw(nu,x), nu0);
            end
        end
        
        function y = Dw(v,x)
            y = pu(-v-1/2,x);
        end
        
        function d = derivativeDw(nu,x)
            d = (Dw(nu+1e-8,x)-Dw(nu-1e-8,x))/2e-8;
        end
        
        function y = npdf(x)
            y = exp(-x.^2/2)/sqrt(2*pi);
        end
        
    end
end % qbp function