function q = qnbm(P, t)
% QNBM inverse CDF of absolute value of Brownian Motion process
% 
% Syntax
%
% q = qnbm(p,t)
%
% Description
% QNBM(Q,T) computes the inverse of the cumulative distribution function of 
% the supremum of the absolute value of the Brownian motion in the interval 
% [0,T] at each of the values in P.
% The input parameter T must be a positive scalar. The values in P must
% probability values.
% The default value of parameter T is 1.
%
% Reference 
% 1- "Convergence of probability measures(2nd ed)" by Billingsley 1999 p.97
% 2- "Empirical processes with applications to statistics"
%    by Shorack, Galen R. and Wellner, Jon A. 1986.
%    expression (7) page 34
% 3- "A toolbox note on the distribution of the maximum of Gaussian process" 
%    by Azais, J.M. & Lozada-Chang, L.

if nargin < 2,
    t = 1;
end
options = optimset('TolX', 1e-5, 'Display', 'off');
q = zeros(size(P));
pk = 1;
for p = P
    q(pk) = fzero(@pnbm, [1e-8, 10], options);
    pk = pk + 1;
end

% NESTED FUNCTION
    function v = pnbm(u)
        % Feller %%%%%%%%%
        ut = u/sqrt(t);
        f1 = 4/pi*exp(-pi^2/(8*ut^2));
        v = f1-p;
        k = 1;
        while abs(f1)> 10^(-8)
            K = 2*k+1;
            f1 = 4/(pi*K)*(-1)^k * exp(-K^2*pi^2/(8*ut^2));
            v = v + f1;
            k = k + 1;
        end
    end
end