function q = qnbp(P, T, n)
% QNBP inverse of CDF of the sup. of abs. val. of the Bessel Process
%
% Syntax
%
% q = qnbp(p,t)
%
% Description
% QNBP(Q,T) computes the inverse of the cumulative distribution function of
% the supremum of the absolute value of the Brownian Bridge in the interval
% [0,T] at each of the values in P.
% The input parameter T must be a positive scalar. The values in P must
% probability values.
% The default value of parameter T is 1.
%
% REMARK    c < 5
%
% Reference
% 1- "Crossing probabilities for a square root boundary by a Bessel
%     process" by DeLong, David M
%     Commun. Stat., Theory Methods A, 1981.
% 2- "A toolbox note on the distribution of the maximum of Gaussian process"
%    by Azais, J.M. & Lozada-Chang, L.
%
% Version 2012/10/11
% Li-Vang Lozada-Chang

if nargin < 3,
    n = 1;
    if nargin < 2,
        T = 2;
    end
end

q = 0*P;
options = optimset('TolX', 1e-5, 'Display', 'iter');
pk = 1;
for p = P
    fun = -1;
    r_C = 1;
    while fun < 0,    % Initial interval
        r_C  = r_C + 1;
        fun = pnbp(r_C);
    end
    q(pk) = fzero(@pnbp, [0.01 r_C], options);
    pk = pk + 1;
end

% NESTED FUNCTION
    function v = pnbp(c)
%        fprintf('.')                % borrar
        Q = n/2;
        bigC = -c^2/2;
        term = 1;
        beta = -1;
        v = 0;
        while (abs(term)>1e-4)
            beta = zeroHGF(beta);
            aux1 = hypergeom(beta + Q, Q + 1, bigC);
            aux2 = derivativeDelongM(beta);
            alph = -(exp(bigC)*(-bigC)^Q *aux1 )/(gamma(Q+1)*beta*aux2);
            term = alph*T^(-beta);
            v = v + term;
        end
        v = v - p;
        %
        function beta = zeroHGF(x0)
            betaPrev = x0;
            beta = x0;
            while (abs(beta - betaPrev) < 1/2)
                x0 = x0 + 1;
                options = optimset('TolX', 1e-5);
                beta = fzero(@delongM, x0);
            end
        end
        %
        function y = delongM(x)
%            fprintf('*')                       % borrar
            y = hypergeom(x + Q, Q, bigC);
        end
        %
        function Dev = derivativeDelongM(x)
            Dev = (delongM(x+1e-8)-delongM(x-1e-8))/(2e-8);
        end     
    end
end     