function Q = qnsc(P,time,omega)
% QNSC inverse of CDF of the sup. of abs. val. of the Sine-Cosine process
%
% Syntax
%   q = qnsc(p,t,omega)
%   q = qnsc(p,t)
%   q = qnsc(p)
%
% Description
% QnSC(P,T,OMEGA) computes the inverse of the cumulative distribution 
% function of the supremum of the absolute value of Sine-Cosine process 
% with parameter OMEGA on [0,T]. T must be a positive scalar 
% The default value of T is 1. The default value of OMEGA is 1.
% The values in P must be probability values. 
%
% REFERENCE:
% "Level Sets and Extrema of Random Processes and Fields" by
% Jean-Marc Azais, Mario Wschebor (2009). Exercise 4.1 p.130
%

% Version 2010/06/09 
% Li-Vang Lozada-Chang
%            

if nargin < 3,
    omega = 1;
    if nargin < 2,
        time = 1;
    end
end

if pi/omega < time,
    Q = sqrt(-2*log(1-P));  %mod JMA 5/7/2018
else
    Q = 0*P; 
    iP = 1;
    for p = P
        options = optimset('TolX', 1e-6, 'Display', 'off');
        u = fzero(@(u) funZero(u), [1e-6, 10], options);
        Q(iP) = u;
        iP = iP + 1;
    end
end
% -------------------------------------------------------------
% NESTED FUNCTION
% -------------------------------------------------------------
    function v = funZero(u)
        %
        v = quadgk(@(t) fun(t), 0, time*omega);
        v = 2*ncdf(u)-1 - time*omega/pi*exp(-u.^2/2)+ v/pi;
        v = v - p;
        
        function y = fun(t)
            % integrand function
            if abs(t) < 1e-10, % avoids null denominator around 0
                y = 0;
            elseif abs(pi - t) < 1e-10, % avoids null denominator around pi
                y = exp(-u^2/2);
            else
                y = exp(-u^2*(1+cos(t))./sin(t).^2);
            end
        end
    end

    function y = ncdf(x)
        % standard normal cdf
        y = erfc(-x ./ sqrt(2))/2;
    end
end